@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with man-db; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * John W. Eaton
  * jwe@che.utexas.edu
@@ -60,12 +60,6 @@ extern char *strstr();
 
 #if defined(HAVE_UNISTD_H)
 #  include <unistd.h>
-#else
-#  ifdef HAVE_GETCWD_H
-extern char *getcwd();
-#  else
-extern char *getwd();
-#  endif
 #endif /* HAVE_UNISTD_H */
 
 #ifndef STDC_HEADERS
@@ -73,29 +67,13 @@ extern char *getenv();
 extern int errno;
 #endif
 
-#if defined(HAVE_LIMITS_H) && defined(_POSIX_VERSION)
-#  include <limits.h>                     /* for PATH_MAX */
-#else /* !(HAVE_LIMITS_H && _POSIX_VERSION) */
-#  include <sys/param.h>                  /* for MAXPATHLEN */
-#endif /* HAVE_LIMITS_H */
-
-#ifndef PATH_MAX
-#  if defined(_POSIX_VERSION) && defined(_POSIX_PATH_MAX)
-#    define PATH_MAX _POSIX_PATH_MAX
-#  else /* !_POSIX_VERSION */
-#    ifdef MAXPATHLEN
-#      define PATH_MAX MAXPATHLEN
-#    else /* !MAXPATHLEN */
-#      define PATH_MAX 1024
-#    endif /* MAXPATHLEN */
-#  endif /* _POSIX_VERSION */
-#endif /* !PATH_MAX */
-
-#include <libintl.h>
+#include "lib/gettext.h"
 #define _(String) gettext (String)
 
 #include "manconfig.h"
 #include "lib/error.h"
+#include "lib/getcwdalloc.h"
+#include "lib/cleanup.h"
 #include "security.h"
 #include "manp.h"
 
@@ -123,7 +101,7 @@ char *manpathlist[MAXDIRS];
 char *user_config_file = NULL;
 
 static void mkcatdirs (const char *mandir, const char *catdir);
-static __inline__ char *get_manpath (char *path);
+static __inline__ char *get_manpath (const char *path);
 static __inline__ char *has_mandir (const char *p);
 static __inline__ char *fsstnd (const char *path);
 static char *def_path (int flag);
@@ -195,9 +173,6 @@ static void print_list (void)
 
 static void add_sections (char *sections)
 {
-	/* No need to free section_list; it's tokenized and each element is
-	 * put into a linked list, which is kept around for later.
-	 */
 	char *section_list = xstrdup (sections);
 	char *sect;
 
@@ -207,6 +182,7 @@ static void add_sections (char *sections)
 		if (debug)
 			fprintf (stderr, "Added section `%s'.\n", sect);
 	}
+	free (section_list);
 }
 
 const char **get_sections (void)
@@ -462,6 +438,7 @@ char *add_nls_manpath (char *manpathlist, const char *locale)
 	/* After doing all the locale stuff we add the manpath to the *END*
 	 * so the locale dirs are checked first on each section */
 	manpath = add_to_manpath (manpath, omanpathlist);
+	free (temp_locale);
 	free (omanpathlist);
 
 	free (manpathlist);
@@ -569,14 +546,15 @@ static char *add_system_manpath (const char *systems, const char *manpathlist)
  */
 static char *guess_manpath (const char *systems)
 {
-	char *path = getenv ("PATH");
+	const char *path = getenv ("PATH");
+	char *manpathlist, *manpath;
 
 	if (path == NULL) {
 		/* Things aren't going to work well, but hey... */
 		if (!quiet)
 			error (0, 0, _("warning: $PATH not set"));
 
-		return add_system_manpath (systems, def_path (MANDATORY));
+		manpathlist = def_path (MANDATORY);
 	} else {
 		if (strlen (path) == 0) {
 			/* Things aren't going to work well here either... */
@@ -587,8 +565,11 @@ static char *guess_manpath (const char *systems)
 						   def_path (MANDATORY));
 		}
 
-		return add_system_manpath (systems, get_manpath (path));
+		manpathlist = get_manpath (path);
 	}
+	manpath = add_system_manpath (systems, manpathlist);
+	free (manpathlist);
+	return manpath;
 }
 
 char *manpath (const char *systems)
@@ -741,7 +722,7 @@ static void add_to_dirlist (FILE *config, int user)
 
 	while ((bp = fgets (buf, BUFSIZ, config))) {
 
-		while (isspace (*bp))
+		while (CTYPE (isspace, *bp))
 			bp++;
 
 		/* TODO: would like a (limited) replacement for sscanf()
@@ -776,11 +757,28 @@ static void add_to_dirlist (FILE *config, int user)
 	}
 }
 
+static void free_config_file (void *unused ATTRIBUTE_UNUSED)
+{
+	struct list *list = namestore, *prev;
+
+	while (list) {
+		free (list->key);
+		free (list->cont);
+		prev = list;
+		list = list->next;
+		free (prev);
+	}
+
+	namestore = tailstore = NULL;
+}
+
 void read_config_file(void)
 {
 	char *home;
 	FILE *config;
 
+	push_cleanup (free_config_file, NULL);
+
 	home = xstrdup (getenv ("HOME"));
 	if (home) {
 		char *dotmanpath;
@@ -857,7 +855,7 @@ static char *def_path (int flag)
  * $HOME/man exists -- the directory $HOME/man will be added
  * to the manpath.
  */
-static __inline__ char *get_manpath (char *path)
+static __inline__ char *get_manpath (const char *path)
 {
 	int len;
 	char *tmppath;
@@ -946,6 +944,8 @@ static __inline__ char *get_manpath (char *path)
 	while (*lp != NULL) {
 		len = strlen (*lp);
 		memcpy (p, *lp, len);
+		free (*lp);
+		*lp = NULL;
 		p += len;
 		*p++ = ':';
 		lp++;
@@ -1028,8 +1028,7 @@ static __inline__ char *has_mandir (const char *path)
 static char **add_dir_to_path_list (char **mphead, char **mp, const char *p)
 {
 	int status;
-	char wd[PATH_MAX];
-	char *cwd = wd;
+	char *cwd;
 
 	if (mp - mphead > MAXDIRS - 1)
 		gripe_overlong_list ();
@@ -1044,12 +1043,11 @@ static char **add_dir_to_path_list (char **mphead, char **mp, const char *p)
 		/* deal with relative paths */
 
 		if (*p != '/') {
-			if (!getcwd (cwd, PATH_MAX - 2 - strlen (p)))
+			cwd = getcwd_allocated ();
+			if (!cwd)
 				error (FATAL, errno,
 				       _("can't determine current directory"));
-			(void) strcat (wd, "/");
-			(void) strcat (wd, p);
-			*mp = xstrdup (wd);
+			*mp = strappend (cwd, "/", p, NULL);
 		} else 
 			*mp = xstrdup (p);
 
@@ -1069,10 +1067,11 @@ void create_pathlist (const char *manp, char **mp)
 
 	for (p = manp;; p = end + 1) {
 		end = strchr (p, ':');
-		if (end)
-			mp = add_dir_to_path_list (mphead, mp,
-						   xstrndup (p, end - p));
-		else {
+		if (end) {
+			char *element = xstrndup (p, end - p);
+			mp = add_dir_to_path_list (mphead, mp, element);
+			free (element);
+		} else {
 			mp = add_dir_to_path_list (mphead, mp, p);
 			break;
 		}
@@ -1080,6 +1079,14 @@ void create_pathlist (const char *manp, char **mp)
 	*mp = NULL;
 }
 
+void free_pathlist (char **mp)
+{
+	while (*mp) {
+		free (*mp);
+		*mp++ = NULL;
+	}
+}
+
 /* Routine to get list of named system and user manpaths (in reverse order). */
 char *get_mandb_manpath (void)
 {
@@ -1116,20 +1123,19 @@ char *get_catpath (const char *name, int cattype)
 			size_t manlen = strlen (list->key);
 			if (STRNEQ (name, list->key, manlen)) {
 				const char *suffix = name + manlen;
-				char *catpath;
+				char *catpath = xstrdup (list->cont);
 
-				catpath = xmalloc (strlen (list->cont) +
-						   strlen (suffix) + 1);
-				(void) strcpy (catpath, list->cont);
 				if (*suffix == '/') {
 					++suffix;
-					(void) strcat (catpath, "/");
+					catpath = strappend (catpath, "/",
+							     NULL);
 				}
-				if (!strncmp (suffix, "man", 3)) {
+				if (STRNEQ (suffix, "man", 3)) {
 					suffix += 3;
-					(void) strcat (catpath, "cat");
+					catpath = strappend (catpath, "cat",
+							     NULL);
 				}
-				(void) strcat (catpath, suffix);
+				catpath = strappend (catpath, suffix, NULL);
 			  	return catpath;
 			}
 		}
