@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with man-db; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /* manp.c */
@@ -27,6 +27,7 @@ extern char *cat_manpath (char *manp);
 extern char *add_nls_manpath (char *manpathlist, const char *locale);
 extern char *manpath (const char *systems);
 extern void create_pathlist (const char *manp, char **mp);
+extern void free_pathlist (char **mp);
 extern char *get_mandb_manpath (void);
 extern char *get_catpath (const char *name, int cattype);
 extern int is_global_mandir (const char *dir);
