@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with man-db; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * code to seek out the original (ultimate) source man file for
  * any specified man file. Soft and hard links and .so inclusions
@@ -97,7 +97,7 @@ extern char *realpath();
 #  endif /* _POSIX_VERSION */
 #endif /* !PATH_MAX */
 
-#include <libintl.h>
+#include "lib/gettext.h"
 #define _(String) gettext (String)
 
 #include "manconfig.h"
